﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MySql.Data.MySqlClient;
using System.Collections;
using parkMonitor.server.CoreThread;

namespace parkMonitor.DataBase
{
    /// <summary>
    /// 数据库连接池
    /// </summary>
    class DBConnectionPool
    {
        private static DBConnectionPool connectionPool = null;//池管理对象
        private static Object objlock = typeof(DBConnectionPool);//池管理对象实例
        private int size = 1;//池中连接数
        private int useCount = 0;//已经使用连接数
        private ArrayList pool = null;//连接保存的集合
        private static string connectionStr = "";//连接字符串

        public DBConnectionPool(string connectionString)
        {
            connectionStr = connectionString;
            pool = new ArrayList();
        }
        /// <summary>
        /// 创建获取池对象
        /// </summary>
        /// <returns></returns>
        public static DBConnectionPool getPool(string connectionString)
        {
            lock (objlock)
            {
                if (connectionPool == null)
                {
                    connectionPool = new DBConnectionPool(connectionString);
                }
                return connectionPool;
            }
        }
        /// <summary>
        /// 获取池连接
        /// </summary>
        /// <returns></returns>
        public MySqlConnection getConnection()
        {
            lock (pool)
            {
                MySqlConnection conn = null;
                if (pool.Count > 0)
                {
                    //取第一个可用连接
                    conn = (MySqlConnection)pool[0];
                    //可用连接中移除
                    pool.RemoveAt(0);
                    //创建不成功
                    if (!isUseful(conn))
                    {
                        useCount--;
                        conn = getConnection();
                    }
                }
                else
                {
                    //可使用连接小于连接数
                    if (useCount <= size)
                    {
                        try
                        {
                            conn = CreateConnection(conn);
                        }
                        catch (Exception)
                        {
                        }
                    }
                }
                if (conn == null)
                {
                    //达到最大连接数递归调用获取连接否则创建新连接
                    if (useCount <= size)
                    {
                        conn = getConnection();
                    }
                    else
                    {
                        conn = CreateConnection(conn);
                    }
                }
                return conn;
            }
        }
        /// <summary>
        /// 创建连接
        /// </summary>
        /// <param name="temp"></param>
        /// <returns></returns>
        private MySqlConnection CreateConnection(MySqlConnection temp)
        {
            MySqlConnection conn = new MySqlConnection(connectionStr);
            useCount++;
            temp = conn;
            return temp;
        }
        /// <summary>
        /// 连接是否创建成功可用
        /// </summary>
        /// <param name="conn"></param>
        /// <returns></returns>
        private bool isUseful(MySqlConnection conn)
        {
            bool result = true;
            if (conn != null)
            {
                string sql = "select garageFreeSpace from garage where garageID = '"+ ParkingSpaceManager.garageID+ "'";
                MySqlCommand cmd = new MySqlCommand(sql, conn);
                try
                {
                    cmd.ExecuteScalar().ToString();
                }
                catch
                {
                    result = false;
                }
            }
            return result;
        }
        /// <summary>
        /// 释放连接，回到连接池
        /// </summary>
        /// <param name="conn"></param>
        public void ConnectionClose(MySqlConnection conn)
        {
            lock (pool)
            {
                if (conn != null)
                {
                    pool.Add(conn);
                }
            }
        }
    }
}
