﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MySql.Data.MySqlClient;
using parkMonitor.server.CoreThread;

namespace parkMonitor.DataBase
{
    public class DBOperation
    {
        /// <summary>
        /// 查询车库表获得剩余车位数
        /// </summary>
        /// <param name="connectionStr">定义本地远端数据库的字符串</param>
        /// <param name="garageID">车库ID</param>
        /// <returns>获得的空闲车位总数</returns>
        public int getGarageFreeSpace(string connectionStr, int garageID)
        {
            MySqlDataReader reader = null;
            int garageFreeSpace = 0;
            string sql = "select * from garage where garageID = '" + garageID + "'";
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0 && reader.Read())
            {
                garageFreeSpace = reader.GetInt32("garageFreeSpace");
                return garageFreeSpace;
            }
            else
            {
                Console.WriteLine("车位剩余数查无结果");
            }
            oper.DBClose(reader);
            return 0;
        }

        /// <summary>
        /// 查询空闲车位位置
        /// </summary>
        /// <param name="connectionStr">定义本地远端数据库的字符串</param>
        /// <param name="garageID">车库ID</param>
        /// <returns>获得空闲车位具体信息</returns>
        public Dictionary<int, Parking_Space> GetParkingSpace(string connectionStr, int garageID)
        {
            Dictionary<int, Parking_Space> lps = new Dictionary<int, Parking_Space>();
            MySqlDataReader reader = null;
            string sql = "select * from parkingspace where parkingSpaceState = 0 and garageID = '" + garageID + "' ";
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0)
            {
                for (int i = 1; i <= count && reader.Read(); i++)
                {
                    Parking_Space ps = new Parking_Space();
                    ps.parkingSpaceID = reader.GetInt32("parkingSpaceID");
                    ps.parkingSpaceX = reader.GetInt32("parkingSpaceX");
                    ps.parkingSpaceY = reader.GetInt32("parkingSpaceY");
                    ps.parkingSpaceZ = reader.GetInt32("parkingSpaceZ");
                    ps.garageID = garageID;
                    lps.Add(ps.parkingSpaceID, (Parking_Space)ps.Clone());
                }
                return lps;
            }
            else
            {
                Console.WriteLine("空闲车位查无结果"); 
            }
            oper.DBClose(reader);
            return null;
        }

        /// <summary>
        /// 查询所有车位位置及状态
        /// </summary>
        /// <param name="connectionStr">定义本地远端数据库的字符串</param>
        /// <param name="garageID">车库ID</param>
        /// <returns>获得所有车位具体信息</returns>
        public Dictionary<int, Parking_Space> GetAllParkingSpace(string connectionStr, int garageID)
        {
            Dictionary<int, Parking_Space> lps = new Dictionary<int, Parking_Space>();
            MySqlDataReader reader = null;
            string sql = "select * from parkingspace where garageID = '" + garageID + "' ";
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0)
            {
                for (int i = 1; i <= count && reader.Read(); i++)
                {
                    Parking_Space ps = new Parking_Space();
                    ps.parkingSpaceID = reader.GetInt32("parkingSpaceID");
                    ps.parkingSpaceX = reader.GetInt32("parkingSpaceX");
                    ps.parkingSpaceY = reader.GetInt32("parkingSpaceY");
                    ps.parkingSpaceZ = reader.GetInt32("parkingSpaceZ");
                    ps.parkingSpaceState = reader.GetInt32("parkingSpaceState");
                    ps.garageID = garageID;
                    lps.Add(ps.parkingSpaceID, (Parking_Space)ps.Clone());
                }
                return lps;
            }
            else
            {
                Console.WriteLine("所有车位查无结果");
            }
            oper.DBClose(reader);
            return null;
        }

        /// <summary>
        /// 数据插入云记录表,并返回停车记录id
        /// </summary>
        /// <param name="connectionStr">定义本地远端数据库的字符串</param>
        /// <param name="userID">用户ID</param>
        /// <param name="numberPlate">号牌</param>
        /// <param name="parkingSpaceID">车位DI</param>
        /// <param name="garageID">车库ID</param>
        /// <param name="parkingRecordsState">停车记录车辆状态</param>
        /// <param name="realParkTime">停车时间</param>
        /// <returns></returns>
        public int InsertToParkingRecords(string connectionStr, int userID, string numberPlate, int parkingSpaceID, int garageID, int parkingRecordsState, string realParkTime)
        {
            string sql = "insert into parkingrecords(userID,numberPlate,parkingSpaceID,garageID,parkingRecordsState,realParkTime) values('" + userID + "','" + numberPlate + "','" + parkingSpaceID + "','" + garageID + "','" + parkingRecordsState + "','" + realParkTime + "')";
            int parkingRecordsID = 0;
            Operation oper = new Operation(connectionStr, sql);
            oper.getInsert();
            parkingRecordsID = oper.getInsertId();

            return parkingRecordsID;
        }

        /// <summary>
        /// 根据车牌号更新车辆表
        /// </summary>
        /// <param name="connectionStr">定义本地远端数据库的字符串</param>
        /// <param name="numberPlate">号牌</param>
        /// <param name="vehiclepParkState">车辆状态</param>
        /// <param name="scanEntryTime">停车开始时间</param>
        /// <param name="parkingRecordsID">停车记录ID</param>
        /// <param name="parkingSpaceID">车位ID</param>
        /// <param name="vehicleTypeConfirm">车辆类型确认</param>
        /// <param name="frontWheelbase">前轮距</param>
        /// <param name="rearWheelbase">后轮距</param>
        public void UpdateVehicle(string connectionStr, string numberPlate, int vehiclepParkState, string scanEntryTime, int parkingRecordsID, int parkingSpaceID, int vehicleTypeConfirm, int frontWheelbase, int rearWheelbase)
        {
            string sql = "update vehicle set vehiclepParkState = '" + vehiclepParkState + "',scanEntryTime = '" + scanEntryTime + "',parkingRecordsID = '" + parkingRecordsID + "',parkingSpaceID = '" + parkingSpaceID + "',vehicleTypeConfirm = '" + vehicleTypeConfirm + "',frontwheelbase = '" + frontWheelbase + "',rearwheelbase = '" + rearWheelbase + "' where numberPlate = '" + numberPlate + "'";
            Operation oper = new Operation(connectionStr, sql);
            oper.getUpdate();

        }

        /// <summary>
        /// 更新车库表剩余车位数
        /// </summary>
        /// <param name="connectionStr">定义本地远端数据库的字符串</param>
        /// <param name="garageFreeSpace">空闲总车位数</param>
        /// <param name="garageID">车库ID</param>
        public void UpdateGarageFreeSpace(string connectionStr, int garageFreeSpace, int garageID)
        {
            string sql = "update garage set garageFreeSpace = '" + garageFreeSpace + "' where garageID = '" + garageID + "'";
            Operation oper = new Operation(connectionStr, sql);
            oper.getUpdate();

        }

        /// <summary>
        /// 根据车牌号更新车辆状态
        /// </summary>
        /// <param name="connectionStr">定义本地远端数据库的字符串</param>
        /// <param name="numberPlate">号牌</param>
        /// <param name="vehiclepParkState">车辆停车状态</param>
        public void UpdateVehicleParkState(string connectionStr, string numberPlate, int vehiclepParkState)
        {
            string sql = "update vehicle set vehiclepParkState = '" + vehiclepParkState + "'where numberPlate = '" + numberPlate + "'";
            Operation oper = new Operation(connectionStr, sql);
            oper.getUpdate();

        }

        /// <summary>
        /// 更新车位状态
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="parkingSpaceID"></param>
        /// <param name="parkingSpaceState"></param>
        public void UpdateParkingSpaceState(string connectionStr, int parkingSpaceID, int parkingSpaceState)
        {
            string sql = "update parkingspace set parkingSpaceState = '" + parkingSpaceState + "'where parkingSpaceID = '" + parkingSpaceID + "'";
            Operation oper = new Operation(connectionStr, sql);
            oper.getUpdate();

        }

        /// <summary>
        /// 更新停车记录表
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="parkingRecordsState"></param>
        /// <param name="realGetTime"></param>
        /// <param name="parkingRecordsID"></param>
        public void UpdateParkingRecords(string connectionStr, int parkingRecordsState, string realGetTime, int parkingRecordsID)
        {
            string sql = "update parkingrecords set parkingRecordsState = '" + parkingRecordsState + "',realGetTime = '" + realGetTime + "'where parkingRecordsID = '" + parkingRecordsID + "'";
            Operation oper = new Operation(connectionStr, sql);
            oper.getUpdate();

        }

        /// <summary>
        /// 插入消息推送表
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="userID"></param>
        /// <param name="context"></param>
        /// <param name="messageType"></param>
        public void InsertToMessageQueue(string connectionStr, int userID, string context, int messageType)
        {
            string sql = "insert into messagequeue(userID,context,messageType) values('" + userID + "','" + context + "','" + messageType + "')";
            Operation oper = new Operation(connectionStr, sql);
            oper.getInsert();

        }

        /// <summary>
        /// 根据车牌查询得到车库id和车位id以及轮距
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="numberPlate"></param>
        /// <returns></returns>
        public Vehicle GetVehicle(string connectionStr, string numberPlate)
        {
            Vehicle v = new Vehicle();
            MySqlDataReader reader = null;
            string sql = "select * from vehicle where numberPlate = '" + numberPlate + "'";
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0 && reader.Read())
            {
                int parkingSpaceID = reader.GetInt32("parkingSpaceID");
                int garageID = reader.GetInt32("garageID");
                int frontwheelbase = reader.GetInt32("frontwheelbase");
                int rearwheelbase = reader.GetInt32("rearwheelbase");
                v.parkingRecordsID = reader.GetInt32("parkingRecordsID");
                v.parkingSpaceID = parkingSpaceID;
                v.garageID = garageID;
                v.frontwheelbase = frontwheelbase;
                v.rearwheelbase = rearwheelbase;
                return v;
            }
            else
            {
                Console.WriteLine("云端车辆查无结果");
            }
            oper.DBClose(reader);
            return null;
        }

        /// <summary>
        /// 根据车位id获得x,y,z
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="parkingSpaceID"></param>
        /// <returns></returns>
        public Parking_Space GetFetchingSpace(string connectionStr, int parkingSpaceID)
        {
            Parking_Space ps = new Parking_Space();
            MySqlDataReader reader = null;
            string sql = "select * from parkingspace where parkingSpaceID = '" + parkingSpaceID + " '";
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0 && reader.Read())
            {
                ps.parkingSpaceID = parkingSpaceID;
                ps.parkingSpaceX = reader.GetInt32("parkingSpaceX");
                ps.parkingSpaceY = reader.GetInt32("parkingSpaceY");
                ps.parkingSpaceZ = reader.GetInt32("parkingSpaceZ");
                return ps;
            }
            else
            {
                Console.WriteLine("车位xyz查无结果");
            }
            oper.DBClose(reader);
            return null;
        }

        /// <summary>
        /// 查询手机号是否被注册
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="tel"></param>
        /// <returns></returns>
        public bool IsTelRegister(string connectionStr, string tel)
        {
            bool isTelRegister = false;
            MySqlDataReader reader = null;
            string sql = "select * from user where userTelephone = '" + tel + "'";
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0)
            {
                isTelRegister = true;
            }
            else
            {
                isTelRegister = false;
            }
            oper.DBClose(reader);
            return isTelRegister;
        }

        /// <summary>
        /// 注册信息写入数据库,返回注册成功信息
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="tel"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public int InsertUser(string connectionStr, string tel, string password)
        {
            string sql = "insert into user(userTelephone,userPassword,userLevel) values('" + tel + "','" + password + "',1)";
            int userID = 0;
            Operation oper = new Operation(connectionStr, sql);
            oper.getInsert();
            userID = oper.getInsertId();

            return userID;
        }

        /// <summary>
        /// 根据电话号码查询userID
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="tel"></param>
        /// <returns></returns>
        public int GetUserID(string connectionStr, string tel)
        {
            int userID = 0;
            MySqlDataReader reader = null;
            string sql = "select userID from user where userTelephone = '" + tel + "'";
            int count = 0;
            Operation oper = new Operation(connectionStr, sql);
            reader = oper.getResultSet(ref count);
            if (count > 0 && reader.Read())
            {
                userID = reader.GetInt32("userID");
                return userID;
            }
            else
            {
                Console.WriteLine("userID查无结果");
            }
            oper.DBClose(reader);
            return 0;
        }

        /// <summary>
        /// 查询停车记录id
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="numberPlate"></param>
        /// <returns></returns>
        public int GetParkingRecordsID(string connectionStr, string numberPlate)
        {
            int parkingRecordsID = 0;
            MySqlDataReader reader = null;
            string sql = "select parkingRecordsID from parkingrecords where numberPlate = '" + numberPlate + "' and parkingRecordsState = 3";
            int count = 0;
            Operation oper = new Operation(connectionStr, sql);
            reader = oper.getResultSet(ref count);
            if (count > 0 && reader.Read())
            {
                parkingRecordsID = reader.GetInt32("parkingRecordsID");
                return parkingRecordsID;
            }
            else
            {
                Console.WriteLine("停车记录id查无结果");
            }
            oper.DBClose(reader);
            return 0;
        }

        /// <summary>
        /// 车库有无此车
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="numberPlate"></param>
        /// <param name="garageID"></param>
        /// <returns></returns>
        public bool IsNumberPlate(string connectionStr, string numberPlate, int garageID)
        {
            bool isNumberPlate = true;
            MySqlDataReader reader = null;
            string sql = "select * from parkingrecords where numberPlate = '" + numberPlate + "' and parkingRecordsState = 3 and garageID = '" + garageID + "'";
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0 && reader.Read())
            {
                isNumberPlate = true;
            }
            else
            {
                isNumberPlate = false;
            }
            oper.DBClose(reader);
            return isNumberPlate;
        }

        /// <summary>
        /// 数据插入本地记录表,并返回停车记录id
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="parkingStatus"></param>
        /// <param name="userID"></param>
        /// <param name="numberPlate"></param>
        /// <param name="parkingSpaceID"></param>
        /// <param name="garageID"></param>
        /// <param name="parkingRecordsState"></param>
        /// <param name="realParkTime"></param>
        /// <param name="frontWheelbase"></param>
        /// <param name="rearWheelbase"></param>
        /// <returns></returns>
        public int InsertToLocalParkingRecords(string connectionStr, int parkingStatus, int userID, string numberPlate, int parkingSpaceID, int garageID, int parkingRecordsState, string realParkTime, int frontWheelbase, int rearWheelbase)
        {
            string sql = "insert into parkingrecords(parkingStatus,userID,numberPlate,parkingSpaceID,garageID,parkingRecordsState,realParkTime,frontWheelbase,rearWheelbase) values('" + parkingStatus + "','" + userID + "','" + numberPlate + "','" + parkingSpaceID + "','" + garageID + "','" + parkingRecordsState + "','" + realParkTime + "','" + frontWheelbase + "','" + rearWheelbase + "')";
            int parkingRecordsID = 0;
            Operation oper = new Operation(connectionStr, sql);
            oper.getInsert();
            parkingRecordsID = oper.getInsertId();

            return parkingRecordsID;
        }

        /// <summary>
        /// 更新本地停车记录表
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="parkingStatus"></param>
        /// <param name="parkingRecordsState"></param>
        /// <param name="realGetTime"></param>
        /// <param name="parkingRecordsID"></param>
        public void UpdateParkingRecords(string connectionStr, int parkingStatus, int parkingRecordsState, string realGetTime, int parkingRecordsID)
        {
            string sql = "update parkingrecords set parkingStatus = '" + parkingStatus + "', parkingRecordsState = '" + parkingRecordsState + "',realGetTime = '" + realGetTime + "'where parkingRecordsID = '" + parkingRecordsID + "'";
            Operation oper = new Operation(connectionStr, sql);
            oper.getUpdate();

        }

        /// <summary>
        /// 插入车辆表
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="numberPlate"></param>
        /// <param name="vehiclepParkState"></param>
        /// <param name="parkingRecordsID"></param>
        /// <param name="parkingSpaceID"></param>
        /// <param name="vehicleTypeConfirm"></param>
        public void InsertVehicle(string connectionStr, string numberPlate, int vehiclepParkState, int parkingRecordsID, int parkingSpaceID, int vehicleTypeConfirm)
        {
            string sql = "insert into vehicle(numberPlate,vehiclepParkState,parkingRecordsID,parkingSpaceID,vehicleTypeConfirm) values('" + numberPlate + "','" + vehiclepParkState + "','" + parkingRecordsID + "','" + parkingSpaceID + "','" + vehicleTypeConfirm + "')";
            Operation oper = new Operation(connectionStr, sql);
            oper.getInsert();

        }

        /// <summary>
        /// 判断车辆表中是否存在该车辆号牌
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="numberPlate"></param>
        /// <returns></returns>
        public bool IsNumberPlateFromVehicle(string connectionStr, string numberPlate)
        {
            bool isNumberPlateFromVehicle = true;
            string sql = "select * from vehicle where numberPlate = '" + numberPlate + "' and vehiclepParkState = 1";
            MySqlDataReader reader = null;
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0 && reader.Read())
            {
                isNumberPlateFromVehicle = true;
            }
            else
            {
                isNumberPlateFromVehicle = false;
            }
            oper.DBClose(reader);
            return isNumberPlateFromVehicle;
        }

        /// <summary>
        /// 根据车牌号更新本地车辆表
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="numberPlate"></param>
        /// <param name="vehiclepParkState"></param>
        /// <param name="parkingRecordsID"></param>
        /// <param name="parkingSpaceID"></param>
        /// <param name="vehicleTypeConfirm"></param>
        public void UpdateLocalVehicle(string connectionStr, string numberPlate, int vehiclepParkState, int parkingRecordsID, int parkingSpaceID, int vehicleTypeConfirm)
        {
            string sql = "update vehicle set vehiclepParkState = '" + vehiclepParkState + "',parkingRecordsID = '" + parkingRecordsID + "',parkingSpaceID = '" + parkingSpaceID + "',vehicleTypeConfirm = '" + vehicleTypeConfirm + "' where numberPlate = '" + numberPlate + "'";
            Operation oper = new Operation(connectionStr, sql);
            oper.getUpdate();

        }

        /// <summary>
        /// 根据车牌查询得到车库id和车位id以及轮距
        /// </summary>
        /// <param name="connectionStr"></param>
        /// <param name="numberPlate"></param>
        /// <param name="garageID"></param>
        /// <returns></returns>
        public Vehicle GetLocalVehicle(string connectionStr, string numberPlate, int garageID)
        {
            Vehicle v = new Vehicle();
            MySqlDataReader reader = null;
            string sql = "select * from parkingrecords where numberPlate = '" + numberPlate + "' and parkingRecordsState = 3 and garageID = '" + garageID + "'";
            Operation oper = new Operation(connectionStr, sql);
            int count = 0;
            reader = oper.getResultSet(ref count);
            if (count > 0 && reader.Read())
            {
                int parkingSpaceID = reader.GetInt32("parkingSpaceID");
                int frontwheelbase = reader.GetInt32("frontwheelbase");
                int rearwheelbase = reader.GetInt32("rearwheelbase");
                v.parkingSpaceID = parkingSpaceID;
                v.garageID = garageID;
                v.frontwheelbase = frontwheelbase;
                v.rearwheelbase = rearwheelbase;
                return v;
            }
            else
            {
                Console.WriteLine("本地车辆查无结果");
            }
            oper.DBClose(reader);
            return null;
        }

    }
}
