﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data;
using MySql.Data.MySqlClient;
using parkMonitor.tools;
using parkMonitor.server.uiLogServer;
using parkMonitor.LOG;

namespace parkMonitor.DataBase
{
    interface IDBOperation
    {
        MySqlDataReader getResultSet(ref int count);
        void getInsert();
        void getUpdate();
        void getDelete();
    }
    public class Operation : IDBOperation
    {
        /// <summary>
        /// 数据库连接异常
        /// </summary>
        public static bool malfunctionRemote = false;
        public static bool malfunctionLocal = false;
        private int count = 0;
        DBConnection connection = null;
        MySqlConnection con = null;
        MySqlCommand cmd = null;
        MySqlDataReader reader = null;
        public Operation(string connectionStr, string sql)
        {
            connection = new DBConnection();
            con = DBConnectionPool.getPool(connectionStr).getConnection();
            cmd = connection.getComm(sql, con);
        }

        /// <summary>
        /// 数据库查询
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public MySqlDataReader getResultSet(ref int count)
        {
            TryOpen(con);
            TryExecute(true, ref count);
            return reader;
        }
        /// <summary>
        /// 数据库插入
        /// </summary>
        public void getInsert()
        {
            TryOpen(con);
            TryExecute(false, ref this.count);
        }
        /// <summary>
        /// 数据库更新
        /// </summary>
        public void getUpdate()
        {
            TryOpen(con);
            TryExecute(false, ref this.count);
        }
        /// <summary>
        /// 数据库删除
        /// </summary>
        public void getDelete()
        {
            TryOpen(con);
            TryExecute(false, ref this.count);
        }
        /// <summary>
        /// 获得插入记录ID
        /// </summary>
        /// <returns></returns>
        public int getInsertId()
        {
            int insertID = 0;
            try
            {
                insertID = Convert.ToInt32(cmd.LastInsertedId);
            }
            catch (Exception ex)
            {
                Console.WriteLine("插入数据库失败");
                Console.WriteLine(ex.Message);
            }
            finally
            {
                DBClose();
            }
            return insertID;
        }

        /// <summary>
        /// 尝试与数据库连接
        /// </summary>
        public static void TryOpen(MySqlConnection conn)
        {
            MyTimer mt = new MyTimer();
            mt.StartTiming();
            int count = 0;
            if (conn == null)
            {
                UILogServer.ins.error("传入非法数据库连接对象,请检查后手动重置数据库状态");
                Log.WriteLog(LogType.NOT_DATABASE, LogFile.ERROR, "传入非法数据库连接对象");
                Operation.malfunctionRemote = true;
                Operation.malfunctionLocal = true;
            }
            else
            {
                while (conn != null && conn.State != ConnectionState.Open)
                {
                    mt.EndTiming();
                    if (mt.IsLonger(45, 1, false, out count) && count >= 2)
                    {
                        if (conn.DataSource == DBConnection.remoteIP)
                        {
                            if (count == 2)
                            {
                                UILogServer.ins.error("暂时无法连接远端数据库, 请检查网络连接后点击“控制面板-启动远端DB”恢复。");
                                Log.WriteLog(LogType.NOT_DATABASE, LogFile.ERROR, "与远端DB失去连接");
                            }
                            Operation.malfunctionRemote = true;
                        }
                        else if (conn.DataSource == DBConnection.localIP)
                        {
                            if (count == 2)
                            {
                                UILogServer.ins.error("暂时无法连接本地数据库,请检查网络连接后点击“控制面板-启动本地DB”恢复。");
                                Log.WriteLog(LogType.NOT_DATABASE, LogFile.ERROR, "与本地DB失去连接");
                            }
                            Operation.malfunctionLocal = true;
                        }
                    }
                    try
                    {
                        conn.Open();
                    }
                    catch (Exception e) { UILogServer.ins.error(e.StackTrace); }
                }
            }
        }

        public static bool MyTransaction(MySqlConnection conn, string sql, out int returnedValue)
        {
            List<string> strs = new List<string>();
            strs.Add(sql);
            return MyTransaction(conn, strs, out returnedValue);
        }

        public static bool MyTransaction(MySqlConnection conn, List<string> sqls, out int returnedValue)
        {
            MySqlTransaction transaction = conn.BeginTransaction();
            MySqlCommand cmd = conn.CreateCommand();
            cmd.Transaction = transaction;
            returnedValue = 0;
            try
            {
                for (int i = 0; i < sqls.Count; i++)
                {
                    cmd.CommandText = sqls[i];
                    cmd.ExecuteNonQuery();
                    returnedValue = Convert.ToInt32(cmd.LastInsertedId);
                }
                transaction.Commit();
                cmd.Dispose();
                //conn.Close();
                //conn.Dispose();
                DBConnectionPool.getPool(DBConnection.remoteConf).ConnectionClose(conn);
                return true;
            }
            catch
            {
                try
                {
                    transaction.Rollback();
                }
                catch { return false; }
                UILogServer.ins.error("数据库操作失败，事务回滚");
                if (cmd != null) { cmd.Dispose(); }
                if (conn != null)
                {
                    //conn.Close(); 
                    //conn.Dispose();
                    DBConnectionPool.getPool(DBConnection.remoteConf).ConnectionClose(conn);
                }
            }
            return false;
        }

        /// <summary>
        /// 尝试执行sql语句
        /// </summary>
        /// <param name="isSearch">是否查询</param>
        /// <param name="count">获得数据条数</param>
        public void TryExecute(bool isSearch, ref int count)
        {
            MyTimer mt = new MyTimer();
            mt.StartTiming();
            int counter = 0;
            while (true)
            {
                mt.EndTiming();
                if (mt.IsLonger(30, 1, false, out counter) && counter >= 1)
                {
                    if (con.DataSource == DBConnection.remoteIP)
                    {
                        if (counter == 1)
                        {
                            UILogServer.ins.error("数据库操作异常，暂停处理自动命令。回滚后点击“启动远端DB”恢复");
                            Log.WriteLog(LogType.NOT_DATABASE, LogFile.ERROR, "操作远端DB异常");
                        }
                        Operation.malfunctionRemote = true;
                    }
                    else if (con.DataSource == DBConnection.localIP)
                    {
                        if (counter == 1)
                        {
                            UILogServer.ins.error("数据库操作异常，暂停处理自动命令。回滚后点击“启动本地DB”恢复");
                            Log.WriteLog(LogType.NOT_DATABASE, LogFile.ERROR, "操作本地DB异常");
                        }
                        Operation.malfunctionLocal = true;
                    }
                    break;
                }
                try
                {
                    //非查询，在内部关闭
                    if (!isSearch)
                    {
                        cmd.ExecuteNonQuery();
                        DBClose();
                        break;
                    }
                    //查询，在上层关闭
                    else
                    {
                        reader = cmd.ExecuteReader();
                        while (reader.Read())
                        {
                            if (reader.HasRows)
                            {
                                count++;
                            }
                        }
                        reader.Close();
                        reader.Dispose();
                        if (count > 0)
                        {
                            reader = cmd.ExecuteReader();
                        }
                        break;
                    }
                }
                catch (Exception)
                {
                    DBClose();
                }
            }
        }

        public static void DBClose(MySqlConnection conn, MySqlCommand cmd, MySqlDataReader reader)
        {
            if (reader != null)
            {
                reader.Close();
                reader.Dispose();
            }
            if (cmd != null)
            {
                cmd.Dispose();
            }
            if (conn != null)
            {
                DBConnectionPool.getPool(DBConnection.remoteConf).ConnectionClose(conn);
            }
        }

        /// <summary>
        /// 关闭连接
        /// </summary>
        public void DBClose()
        {
            DBClose(con, cmd, reader);
        }

        /// <summary>
        /// 关闭连接
        /// </summary>
        /// <param name="reader">需要释放的reader</param>
        public void DBClose(MySqlDataReader reader)
        {
            if (reader != null)
            {
                reader.Close();
                reader.Dispose();
            }
            DBClose();
        }
    }
}
